/* autogenerated from src/include/storage/lwlocklist.h, do not edit */
/* there is deliberately not an #ifndef LWLOCKNAMES_H here */

#define ShmemIndexLock                   (&MainLWLockArray[1].lock)
#define OidGenLock                       (&MainLWLockArray[2].lock)
#define XidGenLock                       (&MainLWLockArray[3].lock)
#define ProcArrayLock                    (&MainLWLockArray[4].lock)
#define SInvalReadLock                   (&MainLWLockArray[5].lock)
#define SInvalWriteLock                  (&MainLWLockArray[6].lock)
#define WALBufMappingLock                (&MainLWLockArray[7].lock)
#define WALWriteLock                     (&MainLWLockArray[8].lock)
#define ControlFileLock                  (&MainLWLockArray[9].lock)
#define MultiXactGenLock                 (&MainLWLockArray[13].lock)
#define RelCacheInitLock                 (&MainLWLockArray[16].lock)
#define CheckpointerCommLock             (&MainLWLockArray[17].lock)
#define TwoPhaseStateLock                (&MainLWLockArray[18].lock)
#define TablespaceCreateLock             (&MainLWLockArray[19].lock)
#define BtreeVacuumLock                  (&MainLWLockArray[20].lock)
#define AddinShmemInitLock               (&MainLWLockArray[21].lock)
#define AutovacuumLock                   (&MainLWLockArray[22].lock)
#define AutovacuumScheduleLock           (&MainLWLockArray[23].lock)
#define SyncScanLock                     (&MainLWLockArray[24].lock)
#define RelationMappingLock              (&MainLWLockArray[25].lock)
#define NotifyQueueLock                  (&MainLWLockArray[27].lock)
#define SerializableXactHashLock         (&MainLWLockArray[28].lock)
#define SerializableFinishedListLock     (&MainLWLockArray[29].lock)
#define SerializablePredicateListLock    (&MainLWLockArray[30].lock)
#define SyncRepLock                      (&MainLWLockArray[32].lock)
#define BackgroundWorkerLock             (&MainLWLockArray[33].lock)
#define DynamicSharedMemoryControlLock   (&MainLWLockArray[34].lock)
#define AutoFileLock                     (&MainLWLockArray[35].lock)
#define ReplicationSlotAllocationLock    (&MainLWLockArray[36].lock)
#define ReplicationSlotControlLock       (&MainLWLockArray[37].lock)
#define CommitTsLock                     (&MainLWLockArray[39].lock)
#define ReplicationOriginLock            (&MainLWLockArray[40].lock)
#define MultiXactTruncationLock          (&MainLWLockArray[41].lock)
#define LogicalRepWorkerLock             (&MainLWLockArray[43].lock)
#define XactTruncationLock               (&MainLWLockArray[44].lock)
#define WrapLimitsVacuumLock             (&MainLWLockArray[46].lock)
#define NotifyQueueTailLock              (&MainLWLockArray[47].lock)
#define WaitEventCustomLock              (&MainLWLockArray[48].lock)
#define WALSummarizerLock                (&MainLWLockArray[49].lock)
#define DSMRegistryLock                  (&MainLWLockArray[50].lock)
#define InjectionPointLock               (&MainLWLockArray[51].lock)
#define SerialControlLock                (&MainLWLockArray[52].lock)
#define AioWorkerSubmissionQueueLock     (&MainLWLockArray[53].lock)

#define NUM_INDIVIDUAL_LWLOCKS		54
